from pyvis.network import Network
import networkx as nx

def generate_network(rows, simplify=True):
    graph = nx.Graph()

    for name in rows:
        graph.add_node(name)

    for name, cons in rows.items():
        for con in cons:
            graph.add_edge(name, con)

    if simplify:
        # Remove isolated nodes and self loops
        graph.remove_nodes_from(list(nx.isolates(graph)))
        graph.remove_edges_from(list(nx.selfloop_edges(graph)))

    # Write networkx graph to GML file
    print("writing GML file")
    nx.readwrite.write_gml(graph, "output.gml")

    net = Network(width="100%", height="100%")
    net.from_nx(graph)
    
    return net




import argparse
import logging
import pathlib
from shutil import which
import subprocess
import sys

import igraph

from snvis import core
from snvis import __version__


def parse_args():
    parser = argparse.ArgumentParser(
        prog="snvis",
        description="Create and view a graph representation of a social network.")
    parser.add_argument(
        "-n",
        type=int,
        help="number of the column containing the name (default: 0)",
        metavar="COLUMN",
        default=0
    )
    parser.add_argument(
        "-c",
        type=int,
        help="number of the column containing the connections (default: 1)",
        metavar="COLUMN",
        default=1
    )
    parser.add_argument(
        "-o",
        type=pathlib.Path,
        help="name of the output file (no extension)",
        metavar="FILE",
        default="graph.svg"
    )
    parser.add_argument(
        "-q",
        help="stop logging to stdout",
        action="store_true"
    )
    parser.add_argument(
        "input",
        type=pathlib.Path,
        help="tab separated values file",
        metavar="SPREADSHEET"
    )
    parser.add_argument(
        "-v",
        "--view",
        help="view the output using default system tool",
        action="store_true"
    )
    parser.add_argument(
        "-s",
        "--simplify",
        help="remove unconnected people and self-loops",
        action="store_true"
    )
    parser.add_argument(
        "--pyvis",
        help="use pyvis instead of igraph",
        action="store_true"
    )
    parser.add_argument(
        "--version",
        help="show the version of snvis that is installed",
        action="version",
        version=f"{parser.prog} {__version__}"
    )
    args = parser.parse_args()
    return args


def main() -> int:
    FORMAT = "[%(levelname)s]: %(message)s"
    logging.basicConfig(level=logging.INFO, format=FORMAT)
    args = parse_args()

    # Argument checking
    if args.n == args.c:
        logging.error("Name and connections columns cannot be equal")
        return 1

    rows = core.parsing.parse_file(
        args.input, args.n, args.c)

    if rows == 1:
        return 1

    if args.pyvis:
        logging.info("Generating network")
        pyvis_net = core.pyvis.generate_network(rows, simplify=args.simplify)
        pyvis_net.show(str(args.o)+".html")
    else:
        graph = core.graphing.generate_igraph(rows, simplify=args.simplify)

        # Output graph
        logging.info("Writing graph to file")
        layout = graph.layout("kk")

        try:
            igraph.plot(graph, str(args.o)+".svg", margin=30, layout=layout)
        except ValueError:
            logging.error(f"Could not write file. Do you have permissions?")
            return 1

        if args.view:
            # Show graph using default program from system
            if sys.platform.startswith("linux"):
                if which("xdg-open"):
                    logging.info("Opening file with xdg-open")
                    subprocess.run(["xdg-open", str(args.o)],
                                   stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
                else:
                    logging.error("xdg-open is not installed")
                    return 1
            # TODO: Following are UNTESTED
            elif sys.platform == "win32":
                logging.info("Opening SVG with default program")
                subprocess.run(["start", str(args.o)])
            elif sys.platform == "darwin":
                logging.info("Opening SVG with default program")
                subprocess.run(["open", str(args.o)])
            else:
                logging.error(f"Unsupported platform: {sys.platform}")
                return 1
    
    logging.info("Exited succesfully")
    return 0


if __name__ == "__main__":
    sys.exit(main())



import logging
import os
from operator import itemgetter

from thefuzz import fuzz


def parse_file(file_path, name_col, cons_col):
    # Check input file exists
    if not os.path.isfile(file_path):
        logging.error(f"Could not find file: '{file_path}'")
        return 1

    # Read spreadsheet
    logging.info("Parsing file")

    rows = {}
    with open(file_path) as f:
        labels = f.readline().strip().split("\t")
        if not name_col < len(labels):
            logging.error(f"Column number '{name_col}' is too large")
            return 1
        elif not cons_col < len(labels):
            logging.error(f"Column number '{cons_col}' is too large")
            return 1
        elif name_col == cons_col:
            logging.error(
                f"Name and connections columns cannot be the same")
            return 1
        for row, line in enumerate(f.readlines()):
            cols = line.strip().split("\t")
            try:
                name = cols[name_col].strip()
            except IndexError:
                logging.error(f"Name column at row {row+2} is empty")
                return 1
            try:
                cons = cols[cons_col].split(",")
                cons = [x.strip() for x in cons]
            except IndexError:
                cons = []
            rows[name] = cons

    def check_name_match(name):
        others = list(rows.keys())
        if name in others:
            return name
        ratios = [(fuzz.ratio(name, to_check), to_check)
                  for to_check in others]
        closest_match = max(ratios, key=itemgetter(0))
        if 100 > closest_match[0] > 80:
            # TODO: Log row
            logging.info(
                f"Replacing {name} with {closest_match[1]}")
            return closest_match[1]
        return None

    for name, cons in rows.items():
        cons = [check_name_match(con) for con in cons]
        cons = [con for con in cons if con != None]
        rows[name] = cons

    return rows



import logging

import igraph


def generate_igraph(rows, simplify=True):
    logging.info("Generating graph")
    people = igraph.Graph()
    people.add_vertices(len(rows))
    people.vs["label"] = list(rows.keys())
    for name, cons in rows.items():
        edge_from = people.vs.find(label=name).index
        for con in cons:
            edge_to = people.vs.find(label=con).index
            people.add_edge(edge_from, edge_to)
            # Stop duplicates
            if name in rows[con]:
                rows[con].remove(name)

    if simplify:
        logging.info("Removing unconnected people and self-loops")
        people.vs.select(_degree=0).delete()
        people.simplify()

    return people
